/* VirtuinoSE example: How to change the value of PWM pin
 * Supported boards: Arduino + Ethernet shield
 * Created by Ilias Lamprou
 * Updated Oct 27 2018
 * Supported App: Virtuino SE
 * More examples: http://virtuino.com/index.php/virtuino-se/20-virtuino-se-code-examples
 */
// ------ VirtuinoSE V Memory Table
//  V1 memory -> Pin 6   PWM output Pin

#include <SPI.h>
#include <Ethernet.h>
//#include <Ethernet2.h>    // enable this line to support Ethernet 2
 
byte mac[] = {0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED};       // Set the ethernet shield mac address.
IPAddress ip(192, 168, 1, 150);                          // Set the ethernet shield ip address. Check your gateway ip address first
EthernetServer server(8000);                             // Server port =8000
 
#define V1_analogOutputPin 6                             // analog output pin
 
//--- VARIABLES
int   V1_analogOutputValueHolder = 0;                

//------------------- USER SETTINGS ---------------------------------
#define VIRTUINO_KEY  "1234"      // Virtuino KEY, only the requests that include this key will be acceptable
                                  // disable the line if you don't want to use a key
//============================================================== setup
//==============================================================
void setup() {
  Serial.begin(9600);
  while (!Serial) {;}
  
  Ethernet.begin(mac, ip);
  server.begin();
  Serial.println("Server IP="+Ethernet.localIP());

  pinMode(V1_analogOutputPin, OUTPUT);   
  analogWrite(V1_analogOutputPin,V1_analogOutputValueHolder);  // set the starting value of analog output        
  }

//============================================================== onCommandReceived
//==============================================================
/* This function is called every time Virtuino app sends a request to server 
 * The 'variableIndex' is the V memory of Virtuino app
 * The 'valueAsText' is the value that has sent from the app
 * If the 'valueAsText" is ? you have to return the value that you want to be displayed on Virtuino panel
 */
 
String onCommandReceived(int variableIndex, String valueAsText){
   Serial.println("=== onCommandReceived  variableIndex="+String(variableIndex)+"   valueAsText="+valueAsText);

  //-------- Control analog output
  if (variableIndex==1){            // Virtuino V1 memory controls the analog output V1_analogOutputPin
      if (valueAsText=="?") return String(V1_analogOutputValueHolder);  // return the value of analog output
      int v=valueAsText.toInt();                      // convert text to number
      V1_analogOutputValueHolder=v;                   // store the new analog value 
      analogWrite(V1_analogOutputPin,V1_analogOutputValueHolder);
   }

  return "";
}


//============================================================== loop
//==============================================================
void loop() {
  
  virtuinoRun();


  }














//================== Virtuino Library (included) ===============
//==============================================================
//==============================================================
/* Only the next two functions are nedded to communicate with Virtuino 
 * Avoid to make changes to the code below  
 */


#define WRONG_KEY_MESSAGE  "Wrong Key"  
#define WELLCOME_MESSAGE  "Hello Virtuino"  
boolean debug=true;
//============================================================== virtuinoRun
//==============================================================
//====================================================================================== run
//======================================================================================
 void virtuinoRun(){
   EthernetClient client = server.available();
  if (client) {
    char c; if (debug) Serial.println("Client connected");
    String lineBuffer;
    
    while (client.connected()) {
      if (client.available()) {
        c = client.read();
        lineBuffer+=c;
        if (debug)  Serial.write(c);
      }
      if (c == '\n' ) {
              int pos = lineBuffer.indexOf("GET /");
              if (pos!=-1){                                   // We have a GET message
                  lineBuffer.remove(0,pos+5);
                  int k= lineBuffer.indexOf(" HTTP/");
                  if (k>0) lineBuffer.remove(k);                       // Clear the tail of the line

                  if (debug) Serial.println("\n\r LineBuffer="+lineBuffer);
                   //---the variable lineBuffer contains the Virtuino command string. 
                   String commandResponse = prepareResponse(&lineBuffer); //Prepare the text that the server has to return to app
              
                  if (debug) Serial.println("\n\r response="+commandResponse);
                  delay(10);
                  client.flush();
                  client.println("HTTP/1.1 200 OK");
                  client.println("Content-Type: text/html");
                  client.println("Connection: close");
                  client.println();
                  client.println(commandResponse);
                  lineBuffer="";
                  break;
                } // if pos
             lineBuffer="";
        } // if c=='\n'
    } // if client.available
    delay(1);
    client.stop();  // close the connection:
    if (debug) Serial.println("Client disconnected");
  }
  }
  

//============================================================== prepareResponse
//==============================================================
String prepareResponse(String* commands){
   
   //---check the KEY
   #ifdef VIRTUINO_KEY 
      String virtuinoKey="";
      int keyPosStart = commands->indexOf("&KEY=");
      if (keyPosStart!=-1) {
        int keyPosEnd = commands->indexOf('&',keyPosStart+5);
        if (keyPosEnd!=-1) virtuinoKey= commands->substring(keyPosStart+5,keyPosEnd);
        else virtuinoKey= commands->substring(keyPosStart);
      }
      if (!virtuinoKey.equals(VIRTUINO_KEY)) return WRONG_KEY_MESSAGE;
   #endif
   
  //--- read all virtuino commands from request
  String commandResponse="";
  boolean checkNext=true;
  int cStart=-1;
  while (checkNext){
    cStart=commands->indexOf('&',cStart+1);
    if (cStart!=-1) {
      int cEnd=commands->indexOf('&',cStart+1);
      String oneCommand="";
      if (cEnd<0) oneCommand=commands->substring(cStart+1); 
      else oneCommand=commands->substring(cStart+1,cEnd); 
      if (debug) Serial.println("cmd: "+oneCommand);
      char commandType = oneCommand.charAt(0);
      if (commandType=='C') return "&CNT="+String(WELLCOME_MESSAGE);               // Virtuino sends the command &C=? to check the connection status   
      if ((commandType=='V') || (commandType=='T')) {
        int equalCharPosition = oneCommand.indexOf('=');
        int variableIndex  = oneCommand.substring(1,equalCharPosition).toInt(); 
        String valueString = oneCommand.substring(equalCharPosition+1); 
        String returnedValue = onCommandReceived(variableIndex,valueString);  
        if (returnedValue.length()>0) {
          commandResponse+='&';
          commandResponse+=commandType;
          commandResponse+=String(variableIndex);
          commandResponse+='=';
          commandResponse+=returnedValue;
        }
      }
    }
    else checkNext=false;   // the last command has checked
  }
  if (debug) Serial.println("Response="+commandResponse );
  if (commandResponse.length()==0) commandResponse="OK";
  return commandResponse;
}


  //=================== ENCODE - DECODE Library (included) ==========
  //=================================================================
  
  //================================================================ urldecode
String urldecode(String* str){
   String encodedString="";
    char c;
    char code0;
    char code1;
    for (int i =0; i < str->length(); i++){
        c=str->charAt(i);
      if (c == '+'){
        encodedString+=' ';  
      }else if (c == '%') {
        i++;
        code0=str->charAt(i);
        i++;
        code1=str->charAt(i);
        c = (h2int(code0) << 4) | h2int(code1);
        encodedString+=c;
      } else{
         encodedString+=c;  
      }
      yield();
    }
    
   return encodedString;
}
//================================================================ urlencode
String urlencode(String* str){
   String encodedString="";
    char c;
    char code0;
    char code1;
    char code2;
    for (int i =0; i < str->length(); i++){
      c=str->charAt(i);
      if (c == ' '){
        encodedString+= '+';
      } else if (isalnum(c)){
        encodedString+=c;
      } else{
        code1=(c & 0xf)+'0';
        if ((c & 0xf) >9){
            code1=(c & 0xf) - 10 + 'A';
        }
        c=(c>>4)&0xf;
        code0=c+'0';
        if (c > 9){
            code0=c - 10 + 'A';
        }
        code2='\0';
        encodedString+='%';
        encodedString+=code0;
        encodedString+=code1;
        //encodedString+=code2;
      }
      yield();
    }
    return encodedString;
}
//================================================================ h2int
unsigned char h2int(char c){
    if (c >= '0' && c <='9'){
        return((unsigned char)c - '0');
    }
    if (c >= 'a' && c <='f'){
        return((unsigned char)c - 'a' + 10);
    }
    if (c >= 'A' && c <='F'){
        return((unsigned char)c - 'A' + 10);
    }
    return(0);
}



